<?php

namespace app\admin\controller;

use app\common\controller\Backend;
use fast\Random;
use think\Exception;
use think\exception\PDOException;
use think\exception\ValidateException;
use think\Validate;

/**
 * 商家员工
 *
 * @icon fa fa-circle-o
 */
class CompanyStaff extends Backend
{

    /**
     * CompanyStaff模型对象
     * @var \app\admin\model\CompanyStaff
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\CompanyStaff;
        $this->view->assign("typeList", $this->model->getTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            //只能看自己的
            $where_op = [];
            if($this->auth->company_id){
                $where_op['company_staff.company_id'] = $this->auth->company_id;
            }

            $list = $this->model
                    ->with(['company'])
                    ->where($where)
                    ->where($where_op)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('company')->visible(['name']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }

    /**
     * 添加
     */
    public function add()
    {
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            $params = $this->preExcludeFields($params);
            if (!$params) {
                $this->error(__('Parameter %s can not be empty', ''));
            }
            $result = false;
            try {
                //是否采用模型验证
                if ($this->modelValidate) {
                    $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                    $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                    $this->model->validateFailException(true)->validate($validate);
                }
                if (isset($params['password'])) {
                    if (!Validate::is($params['password'], "/^[\S]{6,30}$/")) {
                        $this->error(__("Please input correct password"));
                    }
                    $params['salt'] = Random::alnum();
                    $params['password'] = md5(md5($params['password']) . $params['salt']);
                }
                $result = $this->model->allowField(true)->save($params);
            } catch (ValidateException|PDOException|Exception $e) {
                $this->error($e->getMessage());
            }
            if ($result == false) {
               $this->error(__('No rows were inserted'));
            }
            $this->success();
        }
        return $this->view->fetch();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if (!$params) {
                $this->error(__('Parameter %s can not be empty', ''));
            }
            $params = $this->preExcludeFields($params);
            $result = false;
            try {
                //是否采用模型验证
                if ($this->modelValidate) {
                    $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                    $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                    $row->validateFailException(true)->validate($validate);
                }
                if (isset($params['password'])) {
                    if (!Validate::is($params['password'], "/^[\S]{6,30}$/")) {
                        $this->error(__("Please input correct password"));
                    }
                    $params['salt'] = Random::alnum();
                    $params['password'] = md5(md5($params['password']) . $params['salt']);
                }
                $result = $row->allowField(true)->save($params);
            } catch (ValidateException|PDOException|Exception $e) {
                $this->error($e->getMessage());
            }
            if ($result == false) {
                $this->error(__('No rows were updated'));
            }
            $this->success();
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }
}
