<?php

namespace app\admin\controller;

use app\common\controller\Backend;
use think\Db;
/**
 * 用户订单
 *
 * @icon fa fa-circle-o
 */
class Order extends Backend
{

    /**
     * Order模型对象
     * @var \app\admin\model\Order
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\Order;
        $this->view->assign("ordertypeList", $this->model->getOrdertypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("paytypeList", $this->model->getPaytypeList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            //只能看自己的
            $where_op = [];
            if($this->auth->company_id){
                $where_op['order.company_id'] = $this->auth->company_id;
            }

            $list = $this->model
                    ->with(['company','staff','user','preorder','servicetype'])
                    ->where($where)
                    ->where($where_op)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                $row->getRelation('company')->visible(['name']);
				$row->getRelation('staff')->visible(['truename','mobile']);
				$row->getRelation('user')->visible(['nickname','mobile']);
				$row->getRelation('preorder')->visible(['pre_order_no']);
				$row->getRelation('servicetype')->visible(['title']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }

    /**
     * 在线开单
     */
    public function submitordernew(){
        if(!$this->request->post()){
            $this->assign('staff_id',$this->auth->staff_id);
            return $this->view->fetch();
        }

        $data = request_post_hub([
            'user_name','user_car_number','user_mobile','user_address',
            'servicetype_id','server_time','server_info','server_images','pay_fee'
        ]);

        Db::startTrans();


        $data['pre_order_id'] = 0;

        //检查用户
        $user_info = Db::name('user')->where('mobile',$data['user_mobile'])->find();
        if(empty($user_info)){
            Db::rollback();
            $this->error('不存在的用户,请先让客户扫店铺码注册');
        }
        $data['user_id'] = $user_info['id'];

        //检索car_id，没必要了

        //准备数据
        $data['orderno']    = createUniqueNo('O',$user_info['id']);
        $data['ordertype']    = 2;  //类型:1=预约下单,2=在线下单,3=套餐订单
        $data['company_id'] = $this->auth->company_id;
        $data['staff_id'] = $this->auth->staff_id;
        $data['total_fee'] = $data['pay_fee'];

        $data['status'] = 2; //2=已支付,待处理
        $data['createtime'] = time();

        $order_id = Db::name('order')->insertGetId($data);
        if(!$order_id){
            Db::rollback();
            $this->error('下单失败');
        }


        Db::commit();
        $this->success('下单完成',$order_id);
    }

}
