<?php

namespace app\admin\controller;

use app\common\controller\Backend;
use think\Db;
use app\common\service\UserService;
/**
 * 预约单
 *
 * @icon fa fa-circle-o
 */
class Preorder extends Backend
{

    /**
     * Preorder模型对象
     * @var \app\admin\model\Preorder
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\Preorder;
        $this->view->assign("preOrderStatusList", $this->model->getPreOrderStatusList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = true;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            //只能看自己的
            $where_op = [];
            if($this->auth->company_id){
                $where_op['preorder.company_id'] = $this->auth->company_id;
            }

            $list = $this->model
                    ->with(['company','user','servicetype','order'])
                    ->where($where)
                    ->where($where_op)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $row) {
                
                $row->getRelation('company')->visible(['name']);
				$row->getRelation('user')->visible(['nickname','mobile']);
				$row->getRelation('servicetype')->visible(['title']);
				$row->getRelation('order')->visible(['orderno']);
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }

    /**
     * 取消
     */
    public function cancel(){

        $id = input('id',0);
        if(!$this->request->post()){
            $this->assign('id',$id);
            return $this->view->fetch();
        }

        $cancel_reason = input('cancel_reason','');
        $update = [
            'cancel_reason' => $cancel_reason,
            'cancel_time'   => time(),
            'updatetime'    => time(),
            'pre_order_status' => 0,
        ];
        Db::name('pre_order')->where('id',$id)->update($update);
        $userService = new UserService();
        $params['pre_order_id'] = $id;
        $userService->msgPreOrder($params);
        $this->success('取消成功');
    }

    /**
     * 预约开单
     */
    public function submitorder(){
        $pre_order_id = input('id',0); //预约单id
        $map = ['id'=>$pre_order_id];
        $pre_order = Db::name('pre_order')->where($map)->find();
        if(!$this->request->post()){
            $this->assign('id',$pre_order_id);
            $this->assign('row',$pre_order);
            $this->assign('staff_id',$this->auth->staff_id);
            return $this->view->fetch();
        }


        $data = request_post_hub([
            'user_name','user_car_number','user_mobile','user_address',
            'servicetype_id','server_time','server_info','server_images','pay_fee'
        ]);

        Db::startTrans();

        //预约单

        $map = ['id'=>$pre_order_id];
        $pre_order = Db::name('pre_order')->where($map)->lock(true)->find();


        $data['pre_order_id'] = $pre_order_id;
        $data['user_id'] = $pre_order['user_id'];


        //修改状态
        $update = [
            'order_time'   => time(),
            'updatetime'    => time(),
            'pre_order_status' => 2,
        ];
        $update_rs = Db::name('pre_order')->where('id',$pre_order_id)->update($update);
        if($update_rs === false){
            Db::rollback();
            $this->error('下单失败');
        }


        //检索car_id，没必要了

        //准备数据
        $data['orderno']    = createUniqueNo('O',$pre_order_id);
        $data['ordertype']    = 1;  //类型:1=预约下单,2=在线下单,3=套餐订单
        $data['company_id'] = $pre_order['company_id'];
        $data['staff_id']   = $this->auth->staff_id;
        $data['total_fee']  = $data['pay_fee'];

        $data['status'] = 2; //2=已支付,待处理
        $data['createtime'] = time();

        $order_id = Db::name('order')->insertGetId($data);
        if(!$order_id){
            Db::rollback();
            $this->error('下单失败');
        }


        $update = [
            'order_id'   => $order_id,
        ];
        $update_rs = Db::name('pre_order')->where('id',$pre_order_id)->update($update);
        if($update_rs === false){
            Db::rollback();
            $this->error('下单失败');
        }

        Db::commit();
        $this->success('下单完成',$order_id);
    }



}
