<?php

namespace app\common\library;

use app\common\model\CompanyStaff;
use app\common\model\Company;
use app\common\model\UserRule;
use fast\Random;
use think\Config;
use think\Db;
use think\Exception;
use think\Hook;
use think\Request;
use think\Validate;

class Authcompany
{
    protected static $instance = null;
    protected $_error = '';
    protected $_logined = false;
    protected $_user = null;
    protected $_token = '';
    //Token默认有效时长
    protected $keeptime = 2592000;
    protected $requestUri = '';
    protected $rules = [];
    //默认配置
    protected $config = [];
    protected $options = [];
    //protected $allowFields = ['id', 'name', 'logo','image','contacts', 'mobile','province_name','city_name','area_name','province_id','city_id','area_id','address','full_address','longitude','latitude','aptitude_images','open_hours','staff'];
    protected $allowFields = ['id','company_id','type','truename','mobile','company'];


    public function __construct($options = [])
    {
        if ($config = Config::get('company')) {
            $this->config = array_merge($this->config, $config);
        }
        $this->options = array_merge($this->config, $options);
    }

    /**
     *
     * @param array $options 参数
     * @return Auth
     */
    public static function instance($options = [])
    {
        if (is_null(self::$instance)) {
            self::$instance = new static($options);
        }

        return self::$instance;
    }

    /**
     * 生成不重复的随机数字字母组合
     */
    function getUinqueNo($length = 8, $nos = [])
    {
        $newid = Random::build("alnum", $length);
        if (in_array($newid, $nos)) {
            $newid = $this->getUinqueNo($length, $nos);
        }
        return $newid;
    }

    /**
     * 获取User模型
     * @return User
     */
    public function getUser()
    {
        return $this->_user;
    }

    /**
     * 兼容调用user模型的属性
     *
     * @param string $name
     * @return mixed
     */
    public function __get($name)
    {
        return $this->_user ? $this->_user->$name : null;
    }

    /**
     * 兼容调用user模型的属性
     */
    public function __isset($name)
    {
        return isset($this->_user) ? isset($this->_user->$name) : false;
    }

    /**
     * 根据Token初始化
     *
     * @param string $token Token
     * @return boolean
     */
    public function init($token)
    {
        if ($this->_logined) {
            return true;
        }
        if ($this->_error) {
            return false;
        }
        $data = Tokencompany::get($token);
        if (!$data) {
            return false;
        }
        $user_id = intval($data['user_id']);
        if ($user_id > 0) {
            $user = CompanyStaff::get($user_id);
            if (!$user) {
                $this->setError('Account not exist');
                return false;
            }
            if ($user->status != 1) {
                $this->setError('Account is locked');
                return false;
            }

            if(!$user->company_id){
                $this->setError('Account not exist');
                return false;
            }
            $companyinfo = Company::get($user->company_id);
            if(!$companyinfo){
                $this->setError('Account not exist');
                return false;
            }

            $user->company = $companyinfo;

            $this->_user = $user;
            $this->_logined = true;
            $this->_token = $token;

            //初始化成功的事件
            Hook::listen("company_init_successed", $this->_user);

            return true;
        } else {
            $this->setError('You are not logged in');
            return false;
        }
    }


    /**
     * 用户登录
     *
     * @param string $account  账号,用户名、邮箱、手机号
     * @param string $password 密码
     * @return boolean
     */
    public function login($account, $password, $openid)
    {
        $field = 'mobile';
        $user = CompanyStaff::get([$field => $account]);
        if (!$user) {
            $this->setError('Account is incorrect');
            return false;
        }

        if ($user->status != 1) {
            $this->setError('Account is locked');
            return false;
        }
        if ($user->password != $this->getEncryptPassword($password, $user->salt)) {
            $this->setError('Password is incorrect');
            return false;
        }

        //直接登录员工
        return $this->direct($user->id,$openid);
    }

    /**
     * 退出
     *
     * @return boolean
     */
    public function logout()
    {
        if (!$this->_logined) {
            $this->setError('You are not logged in');
            return false;
        }
        //设置登录标识
        $this->_logined = false;
        //删除Token
        Tokencompany::delete($this->_token);
        //退出成功的事件
        Hook::listen("user_logout_successed", $this->_user);
        return true;
    }

    /**
     * 修改密码
     * @param string $newpassword       新密码
     * @param string $oldpassword       旧密码
     * @param bool   $ignoreoldpassword 忽略旧密码
     * @return boolean
     */
    public function resetpwd($newpassword)
    {
        if (!$this->_logined) {
            $this->setError('You are not logged in');
            return false;
        }

        Db::startTrans();

        $salt = Random::alnum();
        $newpassword = $this->getEncryptPassword($newpassword, $salt);
        unset($this->_user['company']);
        $this->_user->save(['password' => $newpassword, 'salt' => $salt]);

        Tokencompany::delete($this->_token);

        Db::commit();

        return true;

    }

    /**
     * 直接登录账号
     * @param int $user_id
     * @return boolean
     */
    public function direct($staff_id,$openid = '')
    {
        $user = CompanyStaff::get($staff_id);
        if ($user) {
            if(!$user->company_id){
                return false;
            }
            $companyinfo = Company::get($user->company_id);
            if(!$companyinfo){
                return false;
            }
            if($companyinfo->status != 1){
                $this->setError('当前门店未通过审核');
                return false;
            }

            Db::startTrans();
            try {

                if(!empty($openid)){
                    $user->openid = $openid;
                    $user->save();
                }
                $user->openid = '';

				$user->company = $companyinfo;// 追加公司

                $this->_user = $user;

                $this->_token = Random::uuid();
                Tokencompany::set($this->_token, $user->id, $this->keeptime);

                $this->_logined = true;

                //登录成功的事件
                Hook::listen("company_login_successed", $this->_user);
                Db::commit();
            } catch (Exception $e) {
                Db::rollback();
                $this->setError($e->getMessage());
                return false;
            }
            return true;
        } else {
            return false;
        }
    }



    /**
     * 判断是否登录
     * @return boolean
     */
    public function isLogin()
    {
        if ($this->_logined) {
            return true;
        }
        return false;
    }

    /**
     * 获取当前Token
     * @return string
     */
    public function getToken()
    {
        return $this->_token;
    }

    /**
     * 获取会员基本信息
     */
    public function getUserinfo()
    {
        $data = $this->_user->toArray();
        $allowFields = $this->getAllowFields();
        $userinfo = array_intersect_key($data, array_flip($allowFields));
        $userinfo = array_merge($userinfo, Tokencompany::get($this->_token));

        //追加
        $userinfo['company']['image'] = one_domain_image($userinfo['company']['image']);
        $userinfo['money'] = model('walletcompany')->getWallet($this->company_id,'money'); //可提现金额
        $userinfo['dongjie_money'] = Db::name('order')->where('company_id',$this->company_id)->where('status',2)->where('ordertype',3)->sum('pay_fee');//冻结金额，即出售的套餐，但没使用的
        $userinfo['all_money'] = bcadd($userinfo['money'],$userinfo['dongjie_money'],2);

        /////////////////////////////////////
        //个人信息
        $strattime = strtotime(date('Y-m-d'));
        $endtime   = $strattime + 86399;



        //今日开单
        $map = [
            'company_id' => $this->company_id,
            'finish_time' => ['BETWEEN',[$strattime,$endtime]], //开单时间用完成时间
            'status'     => 3,
        ];
        if($this->type == 2){
            //员工的
            $map['staff_id'] = $this->id;
        }

        $userinfo['today_order_totalfee'] = Db::name('order')->where($map)->sum('total_fee');;

        //今日新增客户
        $map = [
            'company_id' => $this->company_id,
            'createtime'=>['BETWEEN',[$strattime,$endtime]],
        ];
        if($this->type == 2){
            //员工的
            $map['staff_id'] = $this->id;
        }
        $userinfo['today_newcus'] = Db::name('user_wallet')->where($map)->count();

        //今日新增预约
        $map = [
            'company_id' => $this->company_id,
            'createtime'=>['BETWEEN',[$strattime,$endtime]],
        ];
        $userinfo['today_newpre'] = Db::name('pre_order')->where($map)->count();

        //待处理订单数量
        $map = [
            'company_id' => $this->company_id,
            'status'     => 2,
        ];
        $userinfo['order_unfinish'] = Db::name('order')->where($map)->count();

        //昨日订单数量
        $starttime = strtotime(date('Y-m-d')) - 86400;
        $endtime = strtotime(date('Y-m-d')) - 1;
        $map = [
            'company_id' => $this->company_id,
            'status'     => 3,
            'finish_time'=> ['BETWEEN',[$starttime,$endtime]],
        ];
        $userinfo['order_zuori'] = Db::name('order')->where($map)->count();

        //7日订单数量
        $starttime = strtotime(date('Y-m-d')) - 86400*6;
        $endtime   = strtotime(date('Y-m-d')) + 86399;
        $map = [
            'company_id' => $this->company_id,
            'status'     => 3,
            'finish_time'=> ['BETWEEN',[$starttime,$endtime]],
        ];
        $userinfo['order_qiri'] = Db::name('order')->where($map)->count();


        return $userinfo;
    }



    /**
     * 获取当前请求的URI
     * @return string
     */
    public function getRequestUri()
    {
        return $this->requestUri;
    }

    /**
     * 设置当前请求的URI
     * @param string $uri
     */
    public function setRequestUri($uri)
    {
        $this->requestUri = $uri;
    }

    /**
     * 获取允许输出的字段
     * @return array
     */
    public function getAllowFields()
    {
        return $this->allowFields;
    }

    /**
     * 设置允许输出的字段
     * @param array $fields
     */
    public function setAllowFields($fields)
    {
        $this->allowFields = $fields;
    }



    /**
     * 获取密码加密后的字符串
     * @param string $password 密码
     * @param string $salt     密码盐
     * @return string
     */
    public function getEncryptPassword($password, $salt = '')
    {
        return md5(md5($password) . $salt);
    }

    /**
     * 检测当前控制器和方法是否匹配传递的数组
     *
     * @param array $arr 需要验证权限的数组
     * @return boolean
     */
    public function match($arr = [])
    {
        $request = Request::instance();
        $arr = is_array($arr) ? $arr : explode(',', $arr);
        if (!$arr) {
            return false;
        }
        $arr = array_map('strtolower', $arr);
        // 是否存在
        if (in_array(strtolower($request->action()), $arr) || in_array('*', $arr)) {
            return true;
        }

        // 没找到匹配
        return false;
    }

    /**
     * 设置会话有效时间
     * @param int $keeptime 默认为永久
     */
    public function keeptime($keeptime = 0)
    {
        $this->keeptime = $keeptime;
    }

    /**
     * 渲染用户数据
     * @param array  $datalist  二维数组
     * @param mixed  $fields    加载的字段列表
     * @param string $fieldkey  渲染的字段
     * @param string $renderkey 结果字段
     * @return array
     */
    /*public function render(&$datalist, $fields = [], $fieldkey = 'user_id', $renderkey = 'userinfo')
    {
        $fields = !$fields ? ['id', 'nickname', 'level', 'avatar'] : (is_array($fields) ? $fields : explode(',', $fields));
        $ids = [];
        foreach ($datalist as $k => $v) {
            if (!isset($v[$fieldkey])) {
                continue;
            }
            $ids[] = $v[$fieldkey];
        }
        $list = [];
        if ($ids) {
            if (!in_array('id', $fields)) {
                $fields[] = 'id';
            }
            $ids = array_unique($ids);
            $selectlist = User::where('id', 'in', $ids)->column($fields);
            foreach ($selectlist as $k => $v) {
                $list[$v['id']] = $v;
            }
        }
        foreach ($datalist as $k => &$v) {
            $v[$renderkey] = isset($list[$v[$fieldkey]]) ? $list[$v[$fieldkey]] : null;
        }
        unset($v);
        return $datalist;
    }*/

    /**
     * 设置错误信息
     *
     * @param string $error 错误信息
     * @return Auth
     */
    public function setError($error)
    {
        $this->_error = $error;
        return $this;
    }

    /**
     * 获取错误信息
     * @return string
     */
    public function getError()
    {
        return $this->_error ? __($this->_error) : '';
    }

  
}
